@@ -27,42 +27,44 @@
 
 import os
 import json
+from pathlib import Path
 
 
-DIR_NAME = ".ykman"
+HOME_CONFIG = "~/.ykman"
+XDG_DATA_HOME = os.environ.get("XDG_DATA_HOME", "~/.local/share") + "/ykman"
+XDG_CONFIG_HOME = os.environ.get("XDG_CONFIG_HOME", "~/.config") + "/ykman"
 
-
-def _get_conf_dir():
-    # Note, this is currently experimental and unsupported.
-    # This behavior may be changed at any time.
-    if "YKMAN_XDG_EXPERIMENTAL" in os.environ:
-        # Use XDG base directory structure.
-        xdg_data_default = os.path.join(os.path.expanduser("~"), ".local", "share")
-        xdg_data_home = os.environ.get("XDG_DATA_HOME", xdg_data_default)
-        return os.path.join(xdg_data_home, "ykman")
-
-    return os.path.join(os.path.expanduser("~"), DIR_NAME)
+USE_XDG = "YKMAN_XDG_EXPERIMENTAL" in os.environ
 
 
 class Settings(dict):
+    _config_dir = HOME_CONFIG
+
     def __init__(self, name):
-        self.fname = os.path.join(_get_conf_dir(), name + ".json")
-        if os.path.isfile(self.fname):
-            with open(self.fname, "r") as f:
-                self.update(json.load(f))
+        self.fname = Path(self._config_dir).expanduser().resolve() / (name + ".json")
+        if self.fname.is_file():
+            with self.fname.open("r") as fd:
+                self.update(json.load(fd))
 
     def __eq__(self, other):
         return other is not None and self.fname == other.fname
 
     def __ne__(self, other):
-        return other is not None or self.fname != other.fname
+        return other is None or self.fname != other.fname
 
     def write(self):
-        conf_dir = os.path.dirname(self.fname)
-        if not os.path.isdir(conf_dir):
-            os.makedirs(conf_dir)
-        data = json.dumps(self, indent=2)
-        with open(self.fname, "w") as f:
-            f.write(data)
+        conf_dir = self.fname.parent
+        if not conf_dir.is_dir():
+            conf_dir.mkdir(0o700, parents=True)
+        with self.fname.open("w") as fd:
+            json.dump(self, fd, indent=2)
 
     __hash__ = None
+
+
+class Configuration(Settings):
+    _config_dir = XDG_CONFIG_HOME if USE_XDG else HOME_CONFIG
+
+
+class AppData(Settings):
+    _config_dir = XDG_DATA_HOME if USE_XDG else HOME_CONFIG
